"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @author Olsi Rrjolli
 */
const json_typescript_mapper_1 = require("json-typescript-mapper");
require("reflect-metadata");
const util_1 = require("util");
const constants_1 = require("../constants/constants");
const ValidationError_1 = require("../exceptions/ValidationError");
/**
 * Generate the metakey for the reflection
 *  The Resource Path
 * @param httpMethod
 *  The http method (GET, POST ...)
 * @param path
 * @returns {string}
 *  return metakey to set the reflection id
 */
exports.generateMetaKey = (httpMethod, path = '') => {
    return httpMethod + path;
};
/**
 * Generate the metakey for the reflection
 *  The Resource Path
 * @param httpMethod
 *  The http method (GET, POST ...)
 * @param path
 *  The path
 * @param resource
 *  The resource
 * @returns {string}
 *  return metakey to set the reflection id
 */
exports.generateResourceMetaKey = (httpMethod, resource, path = '') => {
    return httpMethod + resource + path;
};
/**
 * Get the method as function
 * @param controller
 *  The Controller which will contain the method
 * @param toCallenMethodName
 *  The Method name which selected
 * @return Function
 *  The method as Function
 */
exports.getMethod = (controller, toCallenMethodName) => {
    return Reflect.get(controller, toCallenMethodName).bind(controller);
};
/**
 * Get the method name
 * @param requestEvent
 *  The AWS Request Event
 * @param controller
 *  The Controller which will contain the Method.
 * @return {any}
 * The method name
 */
exports.getMethodName = (requestEvent, controller) => {
    return Reflect.getMetadata(exports.generateMetaKey(requestEvent.httpMethod, requestEvent.resource), controller.constructor);
};
/**
 * Get the method meta data
 * @param toCallMethod
 *  The method that contains the MetaData
 * @param toCallController
 *  The Controller which contains the method
 * @return {any}
 *  The metadata as arraz
 */
exports.getMethodMetadata = (toCallMethod, toCallController) => {
    return Reflect.getMetadata(toCallMethod, toCallController, toCallMethod);
};
/**
 * Check if  execution method exist in the Controller
 * @param requestEvent
 *  The Request Event
 * @param controller
 *  The controller
 * @return {boolean}
 *  true if method exist, otherwise false
 */
exports.isMethodExistInController = (requestEvent, controller) => {
    return !util_1.isUndefined(exports.getMethodName(requestEvent, controller));
};
/**
 * Get the method metadata for example for the params and query parameter
 * @param controller
 * The controller for the target
 * @return {any}
 * Metadata as array; can be empty
 */
exports.getControllerMetadata = (controller) => {
    return Reflect.getMetadataKeys(controller.constructor);
};
/**
 * Get the method metadata for example for the params and query parameter
 * @param controller
 * The controller for the target
 * @return {any}
 * Metadata as array; can be empty
 */
exports.getModelForController = (controller) => {
    return Reflect.getOwnMetadata(constants_1.controllerMetaKey.MODEL_KEY, controller.constructor);
};
/**
 * Get the arguments  for the callen method from requestEvent
 * @param awsRequestEvent
 *  The request Event
 * @param toCallMethodName
 *  the method from getting  the arguments
 * @param toCallController
 *  The controller which contains the execution method
 * @returns {string}
 *  return arguments for the method
 */
exports.parseArguments = (awsRequestEvent, toCallMethodName, toCallController) => {
    const methodMetaData = exports.getMethodMetadata(toCallMethodName, toCallController);
    const model = exports.getModelForController(toCallController);
    const args = new Array();
    let bodyIndex = 0;
    if (methodMetaData) {
        // Get Path & Query Params for the to callen method if they set
        methodMetaData.forEach((metaData) => {
            bodyIndex = metaData.index === bodyIndex ? ++bodyIndex : bodyIndex;
            if (awsRequestEvent.pathParameters && awsRequestEvent.pathParameters !== null) {
                args[metaData.index] = awsRequestEvent.pathParameters[metaData.paramName];
            }
            if (awsRequestEvent.queryStringParameters && awsRequestEvent.queryStringParameters !== null) {
                args[metaData.index] = awsRequestEvent.queryStringParameters[metaData.queryName];
            }
        });
    }
    if (!awsRequestEvent.body &&
        (awsRequestEvent.httpMethod === constants_1.httpMethod.POST || awsRequestEvent.httpMethod === constants_1.httpMethod.PUT)) {
        throw new ValidationError_1.default('Invalid data', [{ field: 'body', message: 'body is required' }]);
    }
    args[bodyIndex] = model ? json_typescript_mapper_1.deserialize(model, JSON.parse(awsRequestEvent.body)) : JSON.parse(awsRequestEvent.body);
    return args;
};
//# sourceMappingURL=reflectMetadataHelpers.js.map